/* 
MAT 259A Final Project
Stejara Iulia Dinulescu
Supervisor: George Legrady

Description: 


*/

Table table;
int rowCount;
int NUM_FEATURES = 24;
int NUM_SENSORS = 4;
int NUM_TRIALS = 40;

int feature_for_viz = 0;

ArrayList<Agent> agents = new ArrayList<Agent>();
StringList input_letters = new StringList();
IntList input_counts = new IntList();

import peasy.*;
PeasyCam cam;

import controlP5.*;
ControlP5 cp5;
DropdownList dl;
String input_str = "";

void setup() {
  
  colorMode(HSB, 255);
  //fullScreen(P3D);
  size(1000, 1000, P3D);
  background(0);
  
  cam = new PeasyCam(this, 0, 0, 0, depth*2);
  cam.lookAt(width/2,height/2, depth/2);
  cam.setMaximumDistance(depth * 8);
  cam.setMinimumDistance(-depth);
  
  table = loadTable("viz_features.csv", "header");
  rowCount = table.getRowCount();
  
  String prevl = "A";
  int trial_index = 0; 
  for (int i = 0; i < rowCount; i++) {
    String l = table.getRow(i).getString("Letter");
    if (l.equals(prevl)) { trial_index++; } else { trial_index = 1; }
    prevl = l;
    Agent a = new Agent(l, trial_index); 
    float[] temp_features = new float[NUM_FEATURES];
    int sensor_index = 0; // will go from 1 to NUM_SENSORS
    int feature_index = 0; // will go from 0 to NUM_FEATURES
    for (int j = 1; j <= NUM_FEATURES*NUM_SENSORS; j++) {
      temp_features[feature_index] = table.getRow(i).getFloat(j);
      feature_index++;
      if (feature_index >= NUM_FEATURES) {
        a.features[sensor_index] = temp_features;
        temp_features = new float[NUM_FEATURES]; // clear
        sensor_index++;
        feature_index = 0;
      }
    }
    a.setupMotionParameters();
    agents.add(a);
  }
  
  println("we have " + agents.size() + " agents stored.");
  agents.get(0).printGenotype();
  
  cp5 = new ControlP5(this);
  // create a DropdownList
  dl = cp5.addDropdownList("list1")
          .setPosition(20, 200)
          .addItem("Mean Absolute Deviation", 0)
          .addItem("Peak to Peak", 1)
          .addItem("Spectral Centroid", 2)
          .addItem("Frequency value at maximum magnitude", 3)
          ;
          
  cp5.addTextfield("input")
     .setPosition(20,100)
     .setSize(200,40)
     .setFocus(true)
     .setColor(color(255,0,0))
     ;
  cp5.setAutoDraw(false);
  
  noStroke();
  strokeJoin(ROUND);
  smooth();
}


void draw() {
  
  background(0);
  
  for (int i = 0; i < agents.size(); i++) {
   agents.get(i).display(); 
   connectAgents(i);
   agents.get(i).move();
   agents.get(i).handleBoundaries();
  }
  
  gui();
}

void gui() {
  
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  cp5.draw();
  legend();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST); 
  
}

void legend() {
  pushMatrix();
  noStroke();
  translate(20, height/2-100);
  textSize(50);
  scale(0.3);
  int pos = 50;
  for (int i = 30; i < agents.size(); i+= NUM_TRIALS) {
    fill(agents.get(i).c);
    text(agents.get(i).letter, 0, pos);
    pos += 50;
  }  
  popMatrix();
}

void connectAgents(int index) {
  int rem = index % NUM_TRIALS;
   if ( rem != 0 && agents.get(index).on  && index < agents.size()) {
     pushMatrix();
     stroke(agents.get(index).c); // set the color (property of the agent class)
     
     // search for what letter it is and how many times it appears
     int min = 0; int max = 0;
     if (input_counts.size() > 0) {
       min = input_counts.min();
       max = input_counts.max();
     }
     if (min >= max) { min = 1; max = 3; } 
     for (int i = 0; i < input_letters.size(); i++) {
       if ( agents.get(index).letter.equals(  input_letters.get(i)  )  ) {
         strokeWeight(  map(input_counts.get(i), min, max, 0.5, 3)  );
       }
     }
     line(agents.get(index).position.x, agents.get(index).position.y, agents.get(index).position.z, 
          agents.get(index-1).position.x, agents.get(index-1).position.y, agents.get(index-1).position.z);
     popMatrix(); 
   }
}

void controlEvent(ControlEvent theEvent) {
  if (theEvent.isGroup()) {
    // check if the Event was triggered from a ControlGroup
    println("event from group : "+theEvent.getGroup().getValue()+" from "+theEvent.getGroup());
  } else if (theEvent.isController()) {
    //println("event from controller : "+theEvent.getController().getValue()+" from "+theEvent.getController());
    if (theEvent.getName().equals("list1")) {
      feature_for_viz = (int)theEvent.getController().getValue();
      for (int i = 0; i < agents.size(); i++) {
        agents.get(i).setupMotionParameters();
      }
    }
  } 
}

void keyPressed() {
 if (key == ' ') {
   for (int i = 0; i < agents.size(); i++) {
     agents.get(i).setupMotionParameters();
   }
 }
}

void input(String theText) {
  // automatically receives results from controller input
  input_str = theText.toUpperCase();
  println("input word: " + input_str);
  parseInput();
  
  for (int i = 0; i < agents.size(); i++) {
    agents.get(i).setupMotionParameters();
  }
}

void parseInput() {
  if (!input_str.equals("")) { // if input string is not null
    input_letters.clear();
    input_counts.clear();
    println("input for parsing: " + input_str);
    for (int i = 0; i < agents.size(); i++) { agents.get(i).on = false; } // turn everything off
    for (int i = 0; i < agents.size(); i++) {
      String l = agents.get(i).letter;
       if (input_str.contains(l)) { 
         agents.get(i).on = true; 
       } 
    }
    // only turn on the letters that are in the substring 
    
    for (int i = 0; i < input_str.length(); i++) {
      String l = input_str.substring(i, i+1);
      if (!input_letters.hasValue(l)) {
        // add it to our input_letters string list
        input_letters.append(l);
        input_counts.append(1);
      } else {
        input_counts.increment(  input_letters.index(l)  );
      }
    }
    println(input_letters);
    println(input_counts);
  }
}
